/* ###################################################################
**     Filename    : main.c
**     Processor   : S32K1xx
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.00
** @brief
**         Main module.
**         This module contains user's application code.
*/         
/*!
**  @addtogroup main_module main module documentation
**  @{
*/         
/* MODULE main */


/* Including necessary module. Cpu.h contains other modules needed for compiling.*/
#include "Cpu.h"
#include "pin_mux.h"
#include "clockMan1.h"
#include <stdlib.h>
#include "iseled_driver.h"

#include "main.h"
#include "scg.h"
#include "timer.h"
#include "adc.h"
#include "pcc.h"
#include "lpuart.h"
#include "ets.h"
#include "power_mode.h"
#include "ts_cfg.h"
#include "gpio.h"
#include "gpio_inline_fcn2.h"
#include "flextimer.h"
#include "slider.h"

#include "ets.h"
#include "tcd.h"

#define NR_OF_LEDS      17U

#define LED0            15U
#define LED1            16U
#define LED_GPIO        PTD

/* Timing PAL channel used for 50 Hz LED refresh rate */
#define REFRESH_TIMING_CHANNEL      1UL

/* Period in nanosecond unit */
#define PERIOD_BY_NS  20000000UL /* The period is 20 miliseconds */

typedef struct
{
	uint8_t Red;
	uint8_t Green;
	uint8_t Blue;
	uint16_t Address;
} Set_RGB_Values;

volatile int exit_code = 0;
digLED_InitType testInitType;
uint32_t digLEDResultBufferStrip1[34];
volatile Set_RGB_Values Set_RGB_Params = {0,150,0, 0};
digLED_ReadDataResultType digLEDResultStrip1 = {.chainLength = 17, .retData = &digLEDResultBufferStrip1[0]};
uint64_t counterResolution;

//TS Area
int16_t calibrationGainADC0;
int16_t calibrationGainADC1;
uint8_t  clockMode;
// Low power mode
extern uint8_t  lowPowerModeCtrl, lowPowerModeEnable;
extern tElecStatus  electrodesStatus, sliderElectrodesStatus;
uint8_t  electrodeTouchQualifiedDisplay;
uint8_t  sliderElectrodeTouchQualifiedDisplay;
uint8_t   elecNum, elecNumAct;
// Touch qualification
uint8_t   electrodeTouchQualified[NUMBER_OF_ELECTRODES];
uint8_t   electrodeTouchQualifiedReport;
uint8_t   electrodeTouchNumberPlusOne;

/*****************************************************************************
 *
 * Function: void NVIC_Init(void)
 *
 * Description: Init NVIC
 *
 * Note : IMPORTANT - DMA IRQ MUST BE ENABLED BEFORE LPTMR IRQ!!!
 *****************************************************************************/
void NVIC_Init(void)
{
  // When scanning and reading ADC result data of slider electrodes simultaneously another DMA channel needed
#if (SLIDER_ENABLE && NUMBER_OF_USED_ADC_MODULES == 2)
  NVIC_SET_IRQ_PRIORITY_LEVEL(DMA1_IRQn,1);// set highest priority for dma ch1
  NVIC_SET_IRQ_PRIORITY_LEVEL(DMA2_IRQn,2);// set higher priority for dma ch2
  NVIC_SET_IRQ_PRIORITY_LEVEL(LPTMR0_IRQn,3);// set lower priority for lptmr

  NVIC_IRQ_ENABLE(DMA1_IRQn); // Enable DMA ch1 Interrupts
  NVIC_IRQ_ENABLE(DMA2_IRQn); // Enable DMA ch2 Interrupts
  NVIC_IRQ_ENABLE(LPTMR0_IRQn); // Enable LPTMR0 interrupts in NVIC
#else
  NVIC_SET_IRQ_PRIORITY_LEVEL(DMA1_IRQn,1);// set higher priority for dma
  NVIC_SET_IRQ_PRIORITY_LEVEL(LPTMR0_IRQn,2);// set lower priority for lptmr

  NVIC_IRQ_ENABLE(DMA1_IRQn); // Enable DMA ch1 Interrupts
  NVIC_IRQ_ENABLE(LPTMR0_IRQn); // Enable LPTMR0 interrupts in NVIC
#endif
}

void electrodeGnd(tElecStruct *pElectrodeStruct)
{
  // Drive electrode GPIO low
  REG_RMW32(&(pElectrodeStruct->gpioBasePtr->PCOR),1 << pElectrodeStruct->pinNumberElec,1 << pElectrodeStruct->pinNumberElec);
  // Configure electrode pin as GPIO
  pElectrodeStruct->portBasePtr->PCR[pElectrodeStruct->pinNumberElec] = PORT_PCR_MUX(1);

  // Drive Cext GPIO low
  REG_RMW32(&(pElectrodeStruct->gpioBasePtr->PCOR),1 << pElectrodeStruct->pinNumberCext, 1 << pElectrodeStruct->pinNumberCext);
  // Configure Cext pin as GPIO
  pElectrodeStruct->portBasePtr->PCR[pElectrodeStruct->pinNumberCext] = PORT_PCR_MUX(1);

  // Configure Electrode and Cext pins as outputs at the same time
  REG_WRITE32(&(pElectrodeStruct->gpioBasePtr->PDDR), ((REG_READ32(&(pElectrodeStruct->gpioBasePtr->PDDR))) | (pElectrodeStruct->portMask)));

}

void Iseled_ctrl(void)
{
  // No touch event
  //electrodeTouchQualifiedDisplay = 0xFF;
  static uint8_t previousElectrodeValue[NUMBER_OF_ELECTRODES] = {0xFF, 0xFF};

  if ((previousElectrodeValue[0] != electrodeTouchQualified[0]) || (previousElectrodeValue[1] != electrodeTouchQualified[1]))
  {

    if ((electrodeTouchQualified[0] == 0) && (electrodeTouchQualified[1] == 0))
    {
      Set_RGB_Params.Red = 100;
      Set_RGB_Params.Green = 0;
      Set_RGB_Params.Blue = 0;
      Set_RGB_Params.Address = 0;
      while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);

    }
    else if ((electrodeTouchQualified[0] == 1) && (electrodeTouchQualified[1] == 0))
    {
      Set_RGB_Params.Red = 0;
      Set_RGB_Params.Green = 0;
      Set_RGB_Params.Blue = 100;
      for (int x=2;x<10;x++)
      {
        Set_RGB_Params.Address = x;
        while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);
      }

      Set_RGB_Params.Red = 100;
      Set_RGB_Params.Green = 0;
      Set_RGB_Params.Blue = 0;
      for (int x=10;x<18;x++)
      {
        Set_RGB_Params.Address = x;
        while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);
      }
    }
    else if ((electrodeTouchQualified[0] == 0) && (electrodeTouchQualified[1] == 1))
    {
      Set_RGB_Params.Red = 0;
      Set_RGB_Params.Green = 100;
      Set_RGB_Params.Blue = 0;
      for (int x=10;x<18;x++)
      {
        Set_RGB_Params.Address = x;
        while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);
      }

      Set_RGB_Params.Red = 100;
      Set_RGB_Params.Green = 0;
      Set_RGB_Params.Blue = 0;
      for (int x=2;x<10;x++)
      {
        Set_RGB_Params.Address = x;
        while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);
      }

    }

    else if ((electrodeTouchQualified[0] == 1) && (electrodeTouchQualified[1] == 1))
     {
       Set_RGB_Params.Red = 0;
       Set_RGB_Params.Green = 100;
       Set_RGB_Params.Blue = 0;
       for (int x=10;x<18;x++)
       {
         Set_RGB_Params.Address = x;
         while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);
       }

       Set_RGB_Params.Red = 0;
       Set_RGB_Params.Green = 0;
       Set_RGB_Params.Blue = 100;
       for (int x=2;x<10;x++)
       {
         Set_RGB_Params.Address = x;
         while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);
       }
     }
  }

  previousElectrodeValue[0] = electrodeTouchQualified[0];
  previousElectrodeValue[1] = electrodeTouchQualified[1];

}

/* Delay function - do nothing for a number of cycles */
void delay(volatile int cycles)
{
    while(cycles--);
}
/*****************************************************************************

/* User includes (#include below this line is not maintained by Processor Expert) */

/*! 
  \brief The main function for the project.
  \details The startup initialization sequence is the following:
 * - startup asm routine
 * - main()
*/
int main(void)
{
  /* Write your local variable definition here */

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  #ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT();                   /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of Processor Expert internal initialization.                    ***/

  // Disable interrupts
  DisableInterrupts;
  clockMode = RUN_FIRC;

  testInitType.crcEnable = 1;
  testInitType.firstLedAdr = 1;
  testInitType.tempCmpEnable = 0;
  testInitType.voltSwing = 0;
  testInitType.phaseShift = 0;


  /* ISELED Init */
	CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT, g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
	CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_FORCIBLE);

	//PINS_DRV_Init(N, g_pin_mux_InitConfigArr);

  EDMA_DRV_Init(&dmaController1_State, &dmaController1_InitConfig0, edmaChnStateArray, edmaChnConfigArray, EDMA_CONFIGURED_CHANNELS_COUNT);

	digLED_Init_Interface(NUMBER_OF_INTERFACES, iseled1_InitConfig);

	/*Initialize timing pal instance used by the ISELED Driver for the timeout mechanism*/
	TIMING_Init(&timing_pal1_instance, &timing_pal1_InitConfig);


  delay(100000);
  /* Get tick resolution in microseconds unit for TIMING over LPIT */
  TIMING_GetResolution(&timing_pal1_instance, TIMER_RESOLUTION_TYPE_NANOSECOND, &counterResolution);
  /*Start the timer necessary for the 50fps required by the updating functions (scanColor and dmaBlock)*/
  TIMING_StartChannel(&timing_pal1_instance, REFRESH_TIMING_CHANNEL, PERIOD_BY_NS/counterResolution);

	/* TS init */

	SCG_Init(clockMode);
  //Peripheral clock enable
  PCC_Init(clockMode);

  // Flash and enable I/D cache and write buffer
  LMEM->PCCCR = LMEM_PCCCR_GO_MASK | LMEM_PCCCR_INVW1_MASK | \
      LMEM_PCCCR_INVW0_MASK | LMEM_PCCCR_ENCACHE_MASK;

  // Wake-up timer init
  LPTMR0_Init(LPTMR_ELEC_CAL);

  // ADC0 calibration init
  calibrationGainADC0 = ADC0_Calibration();

  // ADC1 calibration init
  calibrationGainADC1 = ADC1_Calibration();

  // ADC0 init (sample time, samples number to average)
  ADC0_Init(ADC_SWTRIGGER_SAMPLE_TIME, 0, clockMode);

  // ADC1 init (sample time, samples number to average)
  ADC1_Init(ADC_SWTRIGGER_SAMPLE_TIME, 0, clockMode);

  // FTM2 init
  FTM2_Init();

  // Init electrode touch sense
  ElectrodeTouchSenseInit();


#if SLIDER_ENABLE
  // Init slider electrode touch sense
  SliderElectrodeTouchSenseInit();
#endif

#if (TS_ELECTRODE_DATA_ACQUISITION == DMA_ELECTRODE_DATA_ACQUISITION)
    // DMA Touch Sense Engine init
  DMA_init(NUMBER_OF_ELECTRODE_SENSING_PRECYCLES_PER_SAMPLE + numberOfElectrodeSensingCyclesPerSample);
  // Initialize transfer control descriptor for scatter-gather DMA CH0 for touch buttons electrodes
  TCD_Keypads_Init();
#if SLIDER_ENABLE
  // Initialize transfer control descriptor for scatter-gather DMA CH0 for slider electrodes
  TCD_Slider_Init();
#endif
#endif

  // Init NVIC
  NVIC_Init();

  // Enable interrupts
  EnableInterrupts;

  while (digLED_Reset(0) == DIGLED_BUSY);

  while (digLED_Init_Strip(&testInitType, &digLEDResultStrip1, 0) == DIGLED_BUSY);

  Set_RGB_Params.Green = 0;
  Set_RGB_Params.Blue = 0;
  Set_RGB_Params.Red = 0;

  while (digLED_Set_RGB(Set_RGB_Params.Red, Set_RGB_Params.Green, Set_RGB_Params.Blue, Set_RGB_Params.Address, 0) == DIGLED_BUSY);


  while (1)
  {
    // Control RGB LED - touch events display
    Iseled_ctrl();

    // Run low power mode?
    if ((lowPowerModeCtrl == ON) && (lowPowerModeEnable == YES))
    {
      // S32K144EVB Q100 board
      // RUN_FIRC mode typ. 20mA
      // VLPS static mode typ. 30uA

      Run_to_VLPS();
    }

  }

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.1 [05.21]
**     for the NXP S32K series of microcontrollers.
**
** ###################################################################
*/
